@@ -83,9 +83,7 @@ var window = {
   eventListeners: {},
   addEventListener(id, func) {
     var listeners = this.eventListeners[id];
-    if (!listeners) {
-      listeners = this.eventListeners[id] = [];
-    }
+    listeners ||= this.eventListeners[id] = [];
     listeners.push(func);
   },
   removeEventListener(id, func) {
@@ -100,9 +98,7 @@ var window = {
   },
   callEventListeners(id) {
     var listeners = this.eventListeners[id];
-    if (listeners) {
-      listeners.forEach((listener) => listener());
-    }
+    listeners?.forEach((listener) => listener());
   },
   URL: {
     createObjectURL(x) {
@@ -161,17 +157,15 @@ var document = {
   },
   elements: {},
   querySelector(id) {
-    if (!document.elements[id]) {
-      document.elements[id] = {
-        classList: {
-          add() {},
-          remove() {},
-        },
-        eventListeners: {},
-        addEventListener: document.addEventListener,
-        removeEventListener: document.removeEventListener,
-        callEventListeners: document.callEventListeners,
-      };
+    document.elements[id] ||= {
+      classList: {
+        add() {},
+        remove() {},
+      },
+      eventListeners: {},
+      addEventListener: document.addEventListener,
+      removeEventListener: document.removeEventListener,
+      callEventListeners: document.callEventListeners,
     };
     return document.elements[id];
   },
@@ -214,7 +208,7 @@ var XMLHttpRequest = function() {
       } else {
         window.setTimeout(() => {
           this.doSend();
-          if (this.onload) this.onload();
+          this.onload?.();
         }, 0);
       }
     },
@@ -239,7 +233,7 @@ var Image = () => {
     this.complete = true;
     this.width = 64;
     this.height = 64;
-    if (this.onload) this.onload();
+    this.onload?.();
   });
 };
 var Worker = (workerPath) => {
@@ -252,7 +246,7 @@ var Worker = (workerPath) => {
 
   function duplicateJSON(json) {
     function handleTypedArrays(key, value) {
-      if (value && value.toString && value.toString().substring(0, 8) == '[object ' && value.length && value.byteLength) {
+      if (value?.toString && value.toString().substring(0, 8) == '[object ' && value.length && value.byteLength) {
         return Array.prototype.slice.call(value);
       }
       return value;
@@ -293,6 +287,4 @@ if (typeof console == 'undefined') {
 }
 
 // additional setup
-if (!Module['canvas']) {
-  Module['canvas'] = document.getElementById('canvas');
-}
+Module['canvas'] ||= document.getElementById('canvas');
