@@ -207,7 +207,7 @@ addToLibrary({
     },
     loadRemoteEntry: (store, path, callback) => {
       var req = store.get(path);
-      req.onsuccess = (event) => { callback(null, event.target.result); };
+      req.onsuccess = (event) => callback(null, event.target.result);
       req.onerror = (e) => {
         callback(e.target.error);
         e.preventDefault();
@@ -220,7 +220,7 @@ addToLibrary({
         callback(e);
         return;
       }
-      req.onsuccess = () => { callback(null); };
+      req.onsuccess = (event) => callback();
       req.onerror = (e) => {
         callback(e.target.error);
         e.preventDefault();
@@ -228,7 +228,7 @@ addToLibrary({
     },
     removeRemoteEntry: (store, path, callback) => {
       var req = store.delete(path);
-      req.onsuccess = () => { callback(null); };
+      req.onsuccess = (event) => callback();
       req.onerror = (e) => {
         callback(e.target.error);
         e.preventDefault();
