@@ -6,35 +6,26 @@
 
 // Implementation of libuuid creating RFC4122 version 4 random UUIDs.
 
-mergeInto(LibraryManager.library, {
+addToLibrary({
   // Clear a 'compact' UUID.
   uuid_clear__deps: ['$zeroMemory'],
-  uuid_clear: function(uu) {
-    // void uuid_clear(uuid_t uu);
-    zeroMemory(uu, 16);
-  },
+  uuid_clear: (uu) => zeroMemory(uu, 16),
 
   // Compare whether or not two 'compact' UUIDs are the same.
   // Returns an integer less than, equal to, or greater than zero if uu1  is found, respectively, to be
   // lexigraphically  less  than,  equal, or greater than uu2.
   uuid_compare__deps: ['memcmp'],
-  uuid_compare: function(uu1, uu2) {
-    // int uuid_compare(const uuid_t uu1, const uuid_t uu2);
-    return _memcmp(uu1, uu2, 16);
-  },
+  uuid_compare: (uu1, uu2) => _memcmp(uu1, uu2, 16),
 
   // Copies the 'compact' UUID variable from src to dst.
   uuid_copy__deps: ['memcpy'],
-  uuid_copy: function(dst, src) {
-    // void uuid_copy(uuid_t dst, const uuid_t src);
-    _memcpy(dst, src, 16);
-  },
+  uuid_copy: (dst, src) => _memcpy(dst, src, 16),
 
   // Write a RFC4122 version 4 compliant UUID largely based on the method found in
   // http://stackoverflow.com/questions/105034/how-to-create-a-guid-uuid-in-javascript
   // tweaked slightly in order to use the 'compact' UUID form used by libuuid.
   uuid_generate__deps: ['$writeArrayToMemory'],
-  uuid_generate: function(out) {
+  uuid_generate: (out) => {
     // void uuid_generate(uuid_t out);
     var uuid = null;
 
@@ -73,7 +64,7 @@ mergeInto(LibraryManager.library, {
 
   // Compares the value of the supplied 'compact' UUID variable uu to the NULL value.
   // If the value is equal to the NULL UUID, 1 is returned, otherwise 0 is returned.
-  uuid_is_null: function(uu) {
+  uuid_is_null: (uu) => {
     // int uuid_is_null(const uuid_t uu);
     for (var i = 0; i < 4; i++, uu = (uu+4)|0) {
       var val = {{{ makeGetValue('uu', 0, 'i32') }}};
@@ -88,7 +79,7 @@ mergeInto(LibraryManager.library, {
   // the form "%08x-%04x-%04x-%04x-%012x" 36 bytes plus the trailing '\0'.
   // Upon successfully parsing the input string, 0 is returned, and the UUID is stored in the location
   // pointed to by uu, otherwise -1 is returned.
-  uuid_parse: function(inp, uu) {
+  uuid_parse: (inp, uu) => {
     // int uuid_parse(const char *in, uuid_t uu);
     inp = UTF8ToString(inp);
     if (inp.length === 36) {
@@ -112,7 +103,7 @@ mergeInto(LibraryManager.library, {
   // Convert a 'compact' form UUID to a string, if the upper parameter is supplied make the string upper case.
   uuid_unparse__docs: '/** @param {number|boolean=} upper */',
   uuid_unparse__deps: ['$stringToUTF8'],
-  uuid_unparse: function(uu, out, upper) {
+  uuid_unparse: (uu, out, upper) => {
     // void uuid_unparse(const uuid_t uu, char *out);
     var i = 0;
     var uuid = 'xxxx-xx-xx-xx-xxxxxx'.replace(/[x]/g, function(c) {
@@ -127,26 +118,22 @@ mergeInto(LibraryManager.library, {
 
   // Convert a 'compact' form UUID to a lower case string.
   uuid_unparse_lower__deps: ['uuid_unparse'],
-  uuid_unparse_lower: function(uu, out) {
+  uuid_unparse_lower: (uu, out) => {
     // void uuid_unparse_lower(const uuid_t uu, char *out);
     _uuid_unparse(uu, out);
   },
 
   // Convert a 'compact' form UUID to an upper case string.
   uuid_unparse_upper__deps: ['uuid_unparse'],
-  uuid_unparse_upper: function(uu, out) {
+  uuid_unparse_upper: (uu, out) => {
     // void uuid_unparse_upper(const uuid_t uu, char *out);
     _uuid_unparse(uu, out, true);
   },
 
-  uuid_type: function(uu) {
-    // int uuid_type(const uuid_t uu);
-    return {{{ cDefs.UUID_TYPE_DCE_RANDOM }}};
-  },
+  // int uuid_type(const uuid_t uu);
+  uuid_type: (uu) => {{{ cDefs.UUID_TYPE_DCE_RANDOM }}},
 
-  uuid_variant: function(uu) {
-    // int uuid_variant(const uuid_t uu);
-    return {{{ cDefs.UUID_VARIANT_DCE }}};
-  }
+  // int uuid_variant(const uuid_t uu);
+  uuid_variant: (uu) => {{{ cDefs.UUID_VARIANT_DCE }}},
 });
 
