@@ -29,7 +29,7 @@ var WasiLibrary = {
       PThread.terminateAllThreads();
 #endif
 #if expectToReceiveOnModule('onExit')
-      if (Module['onExit']) Module['onExit'](code);
+      Module['onExit']?.(code);
 #endif
       ABORT = true;
     }
@@ -551,7 +551,7 @@ var WasiLibrary = {
       });
     });
 #else
-    if (stream.stream_ops && stream.stream_ops.fsync) {
+    if (stream.stream_ops?.fsync) {
       return stream.stream_ops.fsync(stream);
     }
     return 0; // we can't do anything synchronously; the in-memory FS is already synced to
