@@ -4,7 +4,7 @@
  * SPDX-License-Identifier: MIT
  */
 
-mergeInto(LibraryManager.library, {
+addToLibrary({
   $wasmfsOPFSDirectoryHandles__deps: ['$HandleAllocator'],
   $wasmfsOPFSDirectoryHandles: "new HandleAllocator()",
   $wasmfsOPFSFileHandles__deps: ['$HandleAllocator'],
@@ -53,15 +53,13 @@ mergeInto(LibraryManager.library, {
   },
 
   $wasmfsOPFSCreateAsyncAccessHandle__deps: ['$FileSystemAsyncAccessHandle'],
-  $wasmfsOPFSCreateAsyncAccessHandle: function(fileHandle) {
-    return new FileSystemAsyncAccessHandle(fileHandle);
-  },
+  $wasmfsOPFSCreateAsyncAccessHandle: (fileHandle) => new FileSystemAsyncAccessHandle(fileHandle),
 #endif
 
 #if PTHREADS
   $wasmfsOPFSProxyFinish__deps: ['emscripten_proxy_finish'],
 #endif
-  $wasmfsOPFSProxyFinish: function(ctx) {
+  $wasmfsOPFSProxyFinish: (ctx) => {
     // When using pthreads the proxy needs to know when the work is finished.
     // When used with JSPI the work will be executed in an async block so there
     // is no need to notify when done.
@@ -226,12 +224,12 @@ mergeInto(LibraryManager.library, {
   },
 
   _wasmfs_opfs_free_file__deps: ['$wasmfsOPFSFileHandles'],
-  _wasmfs_opfs_free_file: function(fileID) {
+  _wasmfs_opfs_free_file: (fileID) => {
     wasmfsOPFSFileHandles.free(fileID);
   },
 
   _wasmfs_opfs_free_directory__deps: ['$wasmfsOPFSDirectoryHandles'],
-  _wasmfs_opfs_free_directory: function(dirID) {
+  _wasmfs_opfs_free_directory: (dirID) => {
     wasmfsOPFSDirectoryHandles.free(dirID);
   },
 
@@ -314,7 +312,7 @@ mergeInto(LibraryManager.library, {
   },
 
   _wasmfs_opfs_close_blob__deps: ['$wasmfsOPFSBlobs'],
-  _wasmfs_opfs_close_blob: function(blobID) {
+  _wasmfs_opfs_close_blob: (blobID) => {
     wasmfsOPFSBlobs.free(blobID);
   },
 
@@ -395,7 +393,7 @@ mergeInto(LibraryManager.library, {
   },
 
   _wasmfs_opfs_get_size_blob__deps: ['$wasmfsOPFSBlobs'],
-  _wasmfs_opfs_get_size_blob: function(blobID) {
+  _wasmfs_opfs_get_size_blob: (blobID) => {
     // This cannot fail.
     return wasmfsOPFSBlobs.get(blobID).size;
   },
