@@ -7,11 +7,11 @@
 {{{
   // Helper function to export a symbol on the module object
   // if requested.
-  global.maybeExport = (x) => MODULARIZE && EXPORT_ALL ? `Module['${x}'] = ` : '';
+  globalThis.maybeExport = (x) => MODULARIZE && EXPORT_ALL ? `Module['${x}'] = ` : '';
   // Export to the AudioWorkletGlobalScope the needed variables to access
   // the heap. AudioWorkletGlobalScope is unable to access global JS vars
   // in the compiled main JS file.
-  global.maybeExportIfAudioWorklet = (x) => (MODULARIZE && EXPORT_ALL) || AUDIO_WORKLET ? `Module['${x}'] = ` : '';
+  globalThis.maybeExportIfAudioWorklet = (x) => (MODULARIZE && EXPORT_ALL) || AUDIO_WORKLET ? `Module['${x}'] = ` : '';
   null;
 }}}
 
@@ -23,7 +23,7 @@
 #include "runtime_asan.js"
 #endif
 
-#if ASSERTIONS || SAFE_HEAP
+#if ASSERTIONS
 /** @type {function(*, string=)} */
 function assert(condition, text) {
   if (!condition) throw text;
@@ -73,14 +73,10 @@ function updateMemoryViews() {
 #if SUPPORT_BIG_ENDIAN
   {{{ maybeExport('HEAP_DATA_VIEW') }}} HEAP_DATA_VIEW = new DataView(b);
 #endif
-#if MEMORY64 && MAXIMUM_MEMORY > FOUR_GB
-#include "runtime_view_proxy.js"
-#else
   {{{ maybeExport('HEAP8') }}} HEAP8 = new Int8Array(b);
   {{{ maybeExport('HEAP16') }}} HEAP16 = new Int16Array(b);
   {{{ maybeExport('HEAPU8') }}} HEAPU8 = new Uint8Array(b);
   {{{ maybeExport('HEAPU16') }}} HEAPU16 = new Uint16Array(b);
-#endif
   {{{ maybeExport('HEAP32') }}} HEAP32 = new Int32Array(b);
   {{{ maybeExportIfAudioWorklet('HEAPU32') }}} HEAPU32 = new Uint32Array(b);
   {{{ maybeExportIfAudioWorklet('HEAPF32') }}} HEAPF32 = new Float32Array(b);
@@ -100,9 +96,9 @@ if (!ENVIRONMENT_IS_PTHREAD) {
     Module['mem'] ||
 #endif
     new WebAssembly.Memory({
-      'initial': {{{ INITIAL_MEMORY >>> 16 }}},
+      'initial': {{{ INITIAL_MEMORY / WASM_PAGE_SIZE }}},
 #if SHARED_MEMORY || !ALLOW_MEMORY_GROWTH || MAXIMUM_MEMORY != FOUR_GB
-      'maximum': {{{ (ALLOW_MEMORY_GROWTH && MAXIMUM_MEMORY != FOUR_GB ? MAXIMUM_MEMORY : INITIAL_MEMORY) >>> 16 }}},
+      'maximum': {{{ (ALLOW_MEMORY_GROWTH && MAXIMUM_MEMORY != FOUR_GB ? MAXIMUM_MEMORY : INITIAL_MEMORY) / WASM_PAGE_SIZE }}},
 #endif
 #if SHARED_MEMORY
       'shared': true,
