@@ -23,14 +23,11 @@ function dump(item) {
     return '// ' + JSON.stringify(item, null, '  ').replace(/\n/g, '\n// ');
   } catch (e) {
     const ret = [];
-    for (const i in item) {
-      if (Object.prototype.hasOwnProperty.call(item, i)) {
-        const j = item[i];
-        if (typeof j == 'string' || typeof j == 'number') {
-          ret.push(`${i}: ${j}`);
-        } else {
-          ret.push(`${i}: [?]`);
-        }
+    for (const [i, j] of Object.entries(item)) {
+      if (typeof j == 'string' || typeof j == 'number') {
+        ret.push(`${i}: ${j}`);
+      } else {
+        ret.push(`${i}: [?]`);
       }
     }
     return ret.join(',\n');
@@ -39,8 +36,8 @@ function dump(item) {
   }
 }
 
-global.warnings = false;
-global.currentFile = null;
+globalThis.warnings = false;
+globalThis.currentFile = null;
 
 function errorPrefix() {
   if (currentFile) {
@@ -51,7 +48,7 @@ function errorPrefix() {
 }
 
 function warn(a, msg) {
-  global.warnings = true;
+  globalThis.warnings = true;
   if (!msg) {
     msg = a;
     a = false;
@@ -67,14 +64,14 @@ function warnOnce(a, msg) {
     a = false;
   }
   if (!a) {
-    if (!warnOnce.msgs) warnOnce.msgs = {};
+    warnOnce.msgs ||= {};
     if (msg in warnOnce.msgs) return;
     warnOnce.msgs[msg] = true;
     warn(msg);
   }
 }
 
-global.abortExecution = false;
+globalThis.abortExecution = false;
 
 function error(msg) {
   abortExecution = true;
@@ -105,6 +102,10 @@ function sum(x) {
 // key: checkSig, value: true
 // if it is set, __sig is checked for functions and error is reported
 // if <function name>__sig is missing
+function addToLibrary(obj, options = null) {
+  mergeInto(LibraryManager.library, obj, options);
+}
+
 function mergeInto(obj, other, options = null) {
   if (options) {
     // check for unintended symbol redefinition
@@ -141,22 +142,55 @@ function mergeInto(obj, other, options = null) {
   }
 
   for (const key of Object.keys(other)) {
-    if (key.endsWith('__sig')) {
-      if (obj.hasOwnProperty(key)) {
-        const oldsig = obj[key];
-        const newsig = other[key];
-        if (oldsig == newsig) {
-          warn(`signature redefinition for: ${key}`);
-        } else {
-          error(`signature redefinition for: ${key}. (old=${oldsig} vs new=${newsig})`);
+    if (isDecorator(key)) {
+      if (key.endsWith('__sig')) {
+        if (obj.hasOwnProperty(key)) {
+          const oldsig = obj[key];
+          const newsig = other[key];
+          if (oldsig == newsig) {
+            warn(`signature redefinition for: ${key}`);
+          } else {
+            error(`signature redefinition for: ${key}. (old=${oldsig} vs new=${newsig})`);
+          }
         }
       }
-    }
 
-    if (key.endsWith('__deps')) {
-      const deps = other[key];
-      if (!Array.isArray(deps)) {
-        error(`JS library directive ${key}=${deps.toString()} is of type ${typeof deps}, but it should be an array`);
+      const index = key.lastIndexOf('__');
+      const decoratorName = key.slice(index);
+      const type = typeof other[key];
+
+      // Specific type checking for `__deps` which is expected to be an array
+      // (not just any old `object`)
+      if (decoratorName === '__deps') {
+        const deps = other[key];
+        if (!Array.isArray(deps)) {
+          error(`JS library directive ${key}=${deps.toString()} is of type '${type}', but it should be an array`);
+        }
+        for (let dep of deps) {
+          if (dep && typeof dep !== 'string' && typeof dep !== 'function') {
+            error(`__deps entries must be of type 'string' or 'function' not '${typeof dep}': ${key}`)
+          }
+        }
+      } else {
+        // General type checking for all other decorators
+        const decoratorTypes = {
+          '__sig': 'string',
+          '__proxy': 'string',
+          '__asm': 'boolean',
+          '__inline': 'boolean',
+          '__postset': ['string', 'function'],
+          '__docs': 'string',
+          '__nothrow': 'boolean',
+          '__noleakcheck': 'boolean',
+          '__internal': 'boolean',
+          '__user': 'boolean',
+          '__async': 'boolean',
+          '__i53abi': 'boolean',
+        };
+        const expected = decoratorTypes[decoratorName];
+        if (type !== expected && !expected.includes(type)) {
+          error(`Decorator (${key}} has wrong type. Expected '${expected}' not '${type}'`);
+        }
       }
     }
   }
@@ -199,7 +233,7 @@ function isPowerOfTwo(x) {
 }
 
 /** @constructor */
-function Benchmarker() {
+globalThis.Benchmarker = function() {
   const totals = {};
   const ids = [];
   const lastTime = 0;
@@ -210,7 +244,7 @@ function Benchmarker() {
     }
     lastTime = now;
     ids.push(id);
-    totals[id] = totals[id] || 0;
+    totals[id] ||= 0;
   };
   this.stop = function(id) {
     const now = Date.now();
