@@ -4,6 +4,7 @@
  * University of Illinois/NCSA Open Source License.  Both these licenses can be
  * found in the LICENSE file.
  */
+#include <assert.h>
 #include <emscripten/threading.h>
 #include <emscripten/console.h>
 #include <string.h>
@@ -18,7 +19,7 @@
 
 #if defined(__EMSCRIPTEN_PTHREADS__) && defined(__EMSCRIPTEN_OFFSCREEN_FRAMEBUFFER__)
 
-pthread_key_t currentActiveWebGLContext;
+static pthread_key_t currentActiveWebGLContext;
 pthread_key_t currentThreadOwnsItsWebGLContext;
 static pthread_once_t tlsInit = PTHREAD_ONCE_INIT;
 
@@ -27,8 +28,24 @@ static void InitWebGLTls() {
   pthread_key_create(&currentThreadOwnsItsWebGLContext, NULL);
 }
 
+// When OFFSCREEN_FRAMEBUFFER is enabled the EMSCRIPTEN_WEBGL_CONTEXT_HANDLE
+// is a pointer to a struct with two fields.  See registerContext in
+// library_webgl.js
+typedef struct WebGLContextHandle {
+  uint32_t explicit_swap_control;
+  pthread_t owning_thread;
+} WebGLContextHandle;
+
+static inline pthread_t GetOwningThread(EMSCRIPTEN_WEBGL_CONTEXT_HANDLE handle) {
+  return ((WebGLContextHandle*)handle)->owning_thread;
+}
+
+static inline pthread_t GetCurrentTargetThread() {
+  return GetOwningThread(emscripten_webgl_get_current_context());
+}
+
 EMSCRIPTEN_WEBGL_CONTEXT_HANDLE emscripten_webgl_create_context(const char *target, const EmscriptenWebGLContextAttributes *attributes) {
-  GL_FUNCTION_TRACE(__func__);
+  GL_FUNCTION_TRACE();
   if (!attributes) {
     emscripten_err("emscripten_webgl_create_context: attributes pointer is null!");
     return 0;
@@ -39,19 +56,18 @@ EMSCRIPTEN_WEBGL_CONTEXT_HANDLE emscripten_webgl_create_context(const char *targ
     (attributes->proxyContextToMainThread == EMSCRIPTEN_WEBGL_CONTEXT_PROXY_FALLBACK && !emscripten_supports_offscreencanvas())) {
     EmscriptenWebGLContextAttributes attrs = *attributes;
     attrs.renderViaOffscreenBackBuffer = EM_TRUE;
-    return (EMSCRIPTEN_WEBGL_CONTEXT_HANDLE)emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_III, &emscripten_webgl_do_create_context, target, &attrs);
+    return (EMSCRIPTEN_WEBGL_CONTEXT_HANDLE)emscripten_sync_run_in_main_runtime_thread_ptr(EM_FUNC_SIG_PPP, &emscripten_webgl_do_create_context, target, &attrs);
   } else {
     return emscripten_webgl_do_create_context(target, attributes);
   }
 }
 
 EMSCRIPTEN_RESULT emscripten_webgl_make_context_current(EMSCRIPTEN_WEBGL_CONTEXT_HANDLE context) {
-  GL_FUNCTION_TRACE(__func__);
+  GL_FUNCTION_TRACE();
   if (emscripten_webgl_get_current_context() == context)
     return EMSCRIPTEN_RESULT_SUCCESS;
 
-  void *owningThread = *(void**)(context + 4);
-  if (owningThread == pthread_self()) {
+  if (context && GetOwningThread(context) == pthread_self()) {
     EMSCRIPTEN_RESULT r = emscripten_webgl_make_context_current_calling_thread(context);
     if (r == EMSCRIPTEN_RESULT_SUCCESS) {
       pthread_setspecific(currentActiveWebGLContext, (void*)context);
@@ -59,7 +75,7 @@ EMSCRIPTEN_RESULT emscripten_webgl_make_context_current(EMSCRIPTEN_WEBGL_CONTEXT
     }
     return r;
   } else {
-    EMSCRIPTEN_RESULT r = emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_II, &emscripten_webgl_make_context_current_calling_thread, context);
+    EMSCRIPTEN_RESULT r = emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_IP, &emscripten_webgl_make_context_current_calling_thread, context);
     if (r == EMSCRIPTEN_RESULT_SUCCESS) {
       pthread_setspecific(currentActiveWebGLContext, (void*)context);
       pthread_setspecific(currentThreadOwnsItsWebGLContext, (void*)0);
@@ -74,7 +90,7 @@ EMSCRIPTEN_WEBGL_CONTEXT_HANDLE emscripten_webgl_get_current_context(void) {
 }
 
 EMSCRIPTEN_RESULT emscripten_webgl_commit_frame(void) {
-  GL_FUNCTION_TRACE(__func__);
+  GL_FUNCTION_TRACE();
   if (pthread_getspecific(currentThreadOwnsItsWebGLContext))
     return emscripten_webgl_do_commit_frame();
   else
@@ -90,7 +106,7 @@ static void *memdup(const void *ptr, size_t sz) {
 
 ASYNC_GL_FUNCTION_1(EM_FUNC_SIG_VI, void, glActiveTexture, GLenum);
 ASYNC_GL_FUNCTION_2(EM_FUNC_SIG_VII, void, glAttachShader, GLuint, GLuint);
-VOID_SYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIII, void, glBindAttribLocation, GLuint, GLuint, const GLchar*);
+VOID_SYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIIP, void, glBindAttribLocation, GLuint, GLuint, const GLchar*);
 ASYNC_GL_FUNCTION_2(EM_FUNC_SIG_VII, void, glBindBuffer, GLenum, GLuint);
 ASYNC_GL_FUNCTION_2(EM_FUNC_SIG_VII, void, glBindFramebuffer, GLenum, GLuint);
 ASYNC_GL_FUNCTION_2(EM_FUNC_SIG_VII, void, glBindRenderbuffer, GLenum, GLuint);
@@ -102,7 +118,7 @@ ASYNC_GL_FUNCTION_2(EM_FUNC_SIG_VII, void, glBlendFunc, GLenum, GLenum);
 ASYNC_GL_FUNCTION_4(EM_FUNC_SIG_VIIII, void, glBlendFuncSeparate, GLenum, GLenum, GLenum, GLenum);
 
 void glBufferData(GLenum target, GLsizeiptr size, const void *data, GLenum usage) {
-  GL_FUNCTION_TRACE(__func__);
+  GL_FUNCTION_TRACE();
   if (pthread_getspecific(currentThreadOwnsItsWebGLContext)) {
     emscripten_glBufferData(target, size, data, usage);
     return;
@@ -111,17 +127,17 @@ void glBufferData(GLenum target, GLsizeiptr size, const void *data, GLenum usage
   if (size < 256*1024) { // run small buffer sizes asynchronously by copying - large buffers run synchronously
     void *ptr = memdup(data, size);
     if (ptr || !data) { // glBufferData(data=0) can always be handled asynchronously
-      emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIII, &emscripten_glBufferData, ptr, target, size, ptr, usage);
+      emscripten_dispatch_to_thread(GetCurrentTargetThread(), EM_FUNC_SIG_VIPPI, &emscripten_glBufferData, ptr, target, size, ptr, usage);
       return;
     }
     // Fall through on allocation failure and run synchronously.
   }
 
-  emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIIII, &emscripten_glBufferData, target, size, data, usage);
+  emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIPPI, &emscripten_glBufferData, target, size, data, usage);
 }
 
 void glBufferSubData(GLenum target, GLintptr offset, GLsizeiptr size, const void *data) {
-  GL_FUNCTION_TRACE(__func__);
+  GL_FUNCTION_TRACE();
   if (pthread_getspecific(currentThreadOwnsItsWebGLContext)) {
     emscripten_glBufferSubData(target, offset, size, data);
     return;
@@ -130,7 +146,7 @@ void glBufferSubData(GLenum target, GLintptr offset, GLsizeiptr size, const void
   if (size < 256*1024) { // run small buffer sizes asynchronously by copying - large buffers run synchronously
     void *ptr = memdup(data, size);
     if (ptr || !data) {
-      emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIII, &emscripten_glBufferSubData, ptr, target, offset, size, ptr);
+      emscripten_dispatch_to_thread(GetCurrentTargetThread(), EM_FUNC_SIG_VIIII, &emscripten_glBufferSubData, ptr, target, offset, size, ptr);
       return;
     }
     // Fall through on allocation failure and run synchronously.
@@ -179,37 +195,37 @@ VOID_SYNC_GL_FUNCTION_0(EM_FUNC_SIG_V, void, glFlush); // TODO: THIS COULD POTEN
 ASYNC_GL_FUNCTION_4(EM_FUNC_SIG_VIIII, void, glFramebufferRenderbuffer, GLenum, GLenum, GLenum, GLuint);
 ASYNC_GL_FUNCTION_5(EM_FUNC_SIG_VIIIII, void, glFramebufferTexture2D, GLenum, GLenum, GLenum, GLuint, GLint);
 ASYNC_GL_FUNCTION_1(EM_FUNC_SIG_VI, void, glFrontFace, GLenum);
-VOID_SYNC_GL_FUNCTION_2(EM_FUNC_SIG_VII, void, glGenBuffers, GLsizei, GLuint *);
+VOID_SYNC_GL_FUNCTION_2(EM_FUNC_SIG_VIP, void, glGenBuffers, GLsizei, GLuint *);
 ASYNC_GL_FUNCTION_1(EM_FUNC_SIG_VI, void, glGenerateMipmap, GLenum);
-VOID_SYNC_GL_FUNCTION_2(EM_FUNC_SIG_VII, void, glGenFramebuffers, GLsizei, GLuint *);
-VOID_SYNC_GL_FUNCTION_2(EM_FUNC_SIG_VII, void, glGenRenderbuffers, GLsizei, GLuint *);
-VOID_SYNC_GL_FUNCTION_2(EM_FUNC_SIG_VII, void, glGenTextures, GLsizei, GLuint *);
+VOID_SYNC_GL_FUNCTION_2(EM_FUNC_SIG_VIP, void, glGenFramebuffers, GLsizei, GLuint *);
+VOID_SYNC_GL_FUNCTION_2(EM_FUNC_SIG_VIP, void, glGenRenderbuffers, GLsizei, GLuint *);
+VOID_SYNC_GL_FUNCTION_2(EM_FUNC_SIG_VIP, void, glGenTextures, GLsizei, GLuint *);
 VOID_SYNC_GL_FUNCTION_7(EM_FUNC_SIG_VIIIIIII, void, glGetActiveAttrib, GLuint, GLuint, GLsizei, GLsizei *, GLint *, GLenum *, GLchar *);
 VOID_SYNC_GL_FUNCTION_7(EM_FUNC_SIG_VIIIIIII, void, glGetActiveUniform, GLuint, GLuint, GLsizei, GLsizei *, GLint *, GLenum *, GLchar *);
 VOID_SYNC_GL_FUNCTION_4(EM_FUNC_SIG_VIIII, void, glGetAttachedShaders, GLuint, GLsizei, GLsizei *, GLuint *);
-RET_SYNC_GL_FUNCTION_2(EM_FUNC_SIG_III, GLint, glGetAttribLocation, GLuint, const GLchar *);
+RET_SYNC_GL_FUNCTION_2(EM_FUNC_SIG_IIP, GLint, glGetAttribLocation, GLuint, const GLchar *);
 VOID_SYNC_GL_FUNCTION_2(EM_FUNC_SIG_VII, void, glGetBooleanv, GLenum, GLboolean *);
-VOID_SYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIII, void, glGetBufferParameteriv, GLenum, GLenum, GLint *);
+VOID_SYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIIP, void, glGetBufferParameteriv, GLenum, GLenum, GLint *);
 RET_SYNC_GL_FUNCTION_0(EM_FUNC_SIG_I, GLenum, glGetError);
-VOID_SYNC_GL_FUNCTION_2(EM_FUNC_SIG_VII, void, glGetFloatv, GLenum, GLfloat *);
+VOID_SYNC_GL_FUNCTION_2(EM_FUNC_SIG_VIP, void, glGetFloatv, GLenum, GLfloat *);
 VOID_SYNC_GL_FUNCTION_4(EM_FUNC_SIG_VIIII, void, glGetFramebufferAttachmentParameteriv, GLenum, GLenum, GLenum, GLint *);
-VOID_SYNC_GL_FUNCTION_2(EM_FUNC_SIG_VII, void, glGetIntegerv, GLenum, GLint *);
-VOID_SYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIII, void, glGetProgramiv, GLuint, GLenum, GLint *);
-VOID_SYNC_GL_FUNCTION_4(EM_FUNC_SIG_VIIII, void, glGetProgramInfoLog, GLuint, GLsizei, GLsizei *, GLchar *);
-VOID_SYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIII, void, glGetRenderbufferParameteriv, GLenum, GLenum, GLint *);
-VOID_SYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIII, void, glGetShaderiv, GLuint, GLenum, GLint *);
-VOID_SYNC_GL_FUNCTION_4(EM_FUNC_SIG_VIIII, void, glGetShaderInfoLog, GLuint, GLsizei, GLsizei *, GLchar *);
-VOID_SYNC_GL_FUNCTION_4(EM_FUNC_SIG_VIIII, void, glGetShaderPrecisionFormat, GLenum, GLenum, GLint *, GLint *);
-VOID_SYNC_GL_FUNCTION_4(EM_FUNC_SIG_VIIII, void, glGetShaderSource, GLuint, GLsizei, GLsizei *, GLchar *);
-RET_SYNC_GL_FUNCTION_1(EM_FUNC_SIG_II, const GLubyte *, glGetString, GLenum);
-VOID_SYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIII, void, glGetTexParameterfv, GLenum, GLenum, GLfloat *);
-VOID_SYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIII, void, glGetTexParameteriv, GLenum, GLenum, GLint *);
-VOID_SYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIII, void, glGetUniformfv, GLuint, GLint, GLfloat *);
-VOID_SYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIII, void, glGetUniformiv, GLuint, GLint, GLint *);
-RET_SYNC_GL_FUNCTION_2(EM_FUNC_SIG_III, GLint, glGetUniformLocation, GLuint, const GLchar *);
-VOID_SYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIII, void, glGetVertexAttribfv, GLuint, GLenum, GLfloat *);
-VOID_SYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIII, void, glGetVertexAttribiv, GLuint, GLenum, GLint *);
-VOID_SYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIII, void, glGetVertexAttribPointerv, GLuint, GLenum, void **);
+VOID_SYNC_GL_FUNCTION_2(EM_FUNC_SIG_VIP, void, glGetIntegerv, GLenum, GLint *);
+VOID_SYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIIP, void, glGetProgramiv, GLuint, GLenum, GLint *);
+VOID_SYNC_GL_FUNCTION_4(EM_FUNC_SIG_VIIPP, void, glGetProgramInfoLog, GLuint, GLsizei, GLsizei *, GLchar *);
+VOID_SYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIIP, void, glGetRenderbufferParameteriv, GLenum, GLenum, GLint *);
+VOID_SYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIIP, void, glGetShaderiv, GLuint, GLenum, GLint *);
+VOID_SYNC_GL_FUNCTION_4(EM_FUNC_SIG_VIIPP, void, glGetShaderInfoLog, GLuint, GLsizei, GLsizei *, GLchar *);
+VOID_SYNC_GL_FUNCTION_4(EM_FUNC_SIG_VIIPP, void, glGetShaderPrecisionFormat, GLenum, GLenum, GLint *, GLint *);
+VOID_SYNC_GL_FUNCTION_4(EM_FUNC_SIG_VIIPP, void, glGetShaderSource, GLuint, GLsizei, GLsizei *, GLchar *);
+RET_PTR_SYNC_GL_FUNCTION_1(EM_FUNC_SIG_PI, const GLubyte *, glGetString, GLenum);
+VOID_SYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIIP, void, glGetTexParameterfv, GLenum, GLenum, GLfloat *);
+VOID_SYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIIP, void, glGetTexParameteriv, GLenum, GLenum, GLint *);
+VOID_SYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIIP, void, glGetUniformfv, GLuint, GLint, GLfloat *);
+VOID_SYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIIP, void, glGetUniformiv, GLuint, GLint, GLint *);
+RET_SYNC_GL_FUNCTION_2(EM_FUNC_SIG_IIP, GLint, glGetUniformLocation, GLuint, const GLchar *);
+VOID_SYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIIP, void, glGetVertexAttribfv, GLuint, GLenum, GLfloat *);
+VOID_SYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIIP, void, glGetVertexAttribiv, GLuint, GLenum, GLint *);
+VOID_SYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIIP, void, glGetVertexAttribPointerv, GLuint, GLenum, void **);
 ASYNC_GL_FUNCTION_2(EM_FUNC_SIG_VII, void, glHint, GLenum, GLenum);
 RET_SYNC_GL_FUNCTION_1(EM_FUNC_SIG_II, GLboolean, glIsBuffer, GLuint);
 RET_SYNC_GL_FUNCTION_1(EM_FUNC_SIG_II, GLboolean, glIsEnabled, GLenum);
@@ -222,13 +238,13 @@ ASYNC_GL_FUNCTION_1(EM_FUNC_SIG_VF, void, glLineWidth, GLfloat);
 ASYNC_GL_FUNCTION_1(EM_FUNC_SIG_VI, void, glLinkProgram, GLuint);
 ASYNC_GL_FUNCTION_2(EM_FUNC_SIG_VII, void, glPixelStorei, GLenum, GLint);
 ASYNC_GL_FUNCTION_2(EM_FUNC_SIG_VFF, void, glPolygonOffset, GLfloat, GLfloat);
-VOID_SYNC_GL_FUNCTION_7(EM_FUNC_SIG_VIIIIIII, void, glReadPixels, GLint, GLint, GLsizei, GLsizei, GLenum, GLenum, void *);
+VOID_SYNC_GL_FUNCTION_7(EM_FUNC_SIG_VIIIIIIP, void, glReadPixels, GLint, GLint, GLsizei, GLsizei, GLenum, GLenum, void *);
 ASYNC_GL_FUNCTION_0(EM_FUNC_SIG_V, void, glReleaseShaderCompiler);
 ASYNC_GL_FUNCTION_4(EM_FUNC_SIG_VIIII, void, glRenderbufferStorage, GLenum, GLenum, GLsizei, GLsizei);
 ASYNC_GL_FUNCTION_2(EM_FUNC_SIG_VII, void, glSampleCoverage, GLfloat, GLboolean);
 ASYNC_GL_FUNCTION_4(EM_FUNC_SIG_VIIII, void, glScissor, GLint, GLint, GLsizei, GLsizei);
 VOID_SYNC_GL_FUNCTION_5(EM_FUNC_SIG_VIIIII, void, glShaderBinary, GLsizei, const GLuint *, GLenum, const void *, GLsizei);
-VOID_SYNC_GL_FUNCTION_4(EM_FUNC_SIG_VIIII, void, glShaderSource, GLuint, GLsizei, const GLchar *const*, const GLint *);
+VOID_SYNC_GL_FUNCTION_4(EM_FUNC_SIG_VIIPP, void, glShaderSource, GLuint, GLsizei, const GLchar *const*, const GLint *);
 ASYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIII, void, glStencilFunc, GLenum, GLint, GLuint);
 ASYNC_GL_FUNCTION_4(EM_FUNC_SIG_VIIII, void, glStencilFuncSeparate, GLenum, GLenum, GLint, GLuint);
 ASYNC_GL_FUNCTION_1(EM_FUNC_SIG_VI, void, glStencilMask, GLuint);
@@ -256,7 +272,7 @@ static ssize_t ImageSize(int width, int height, GLenum format, GLenum type) {
 }
 
 void glTexImage2D(GLenum target, GLint level, GLint internalformat, GLsizei width, GLsizei height, GLint border, GLenum format, GLenum type, const void *pixels) {
-  GL_FUNCTION_TRACE(__func__);
+  GL_FUNCTION_TRACE();
   if (pthread_getspecific(currentThreadOwnsItsWebGLContext)) {
     emscripten_glTexImage2D(target, level, internalformat, width, height, border, format, type, pixels);
     return;
@@ -266,13 +282,13 @@ void glTexImage2D(GLenum target, GLint level, GLint internalformat, GLsizei widt
   if (!pixels || (sz >= 0 && sz < 256*1024)) { // run small buffer sizes asynchronously by copying - large buffers run synchronously
     void *ptr = memdup(pixels, sz);
     if (ptr || !pixels) {
-      emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIIIIIIII, &emscripten_glTexImage2D, ptr, target, level, internalformat, width, height, border, format, type, ptr);
+     emscripten_dispatch_to_thread(GetCurrentTargetThread(), EM_FUNC_SIG_VIIIIIIIIP, &emscripten_glTexImage2D, ptr, target, level, internalformat, width, height, border, format, type, ptr);
       return;
     }
     // Fall through on allocation failure and run synchronously.
   }
 
-  emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIIIIIIIII, &emscripten_glTexImage2D, target, level, internalformat, width, height, border, format, type, pixels);
+  emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIIIIIIIIP, &emscripten_glTexImage2D, target, level, internalformat, width, height, border, format, type, pixels);
 }
 
 ASYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIIF, void, glTexParameterf, GLenum, GLenum, GLfloat);
@@ -281,7 +297,7 @@ ASYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIII, void, glTexParameteri, GLenum, GLenum, GLi
 VOID_SYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIII, void, glTexParameteriv, GLenum, GLenum, const GLint *);
 
 void glTexSubImage2D(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLsizei width, GLsizei height, GLenum format, GLenum type, const void *pixels) {
-  GL_FUNCTION_TRACE(__func__);
+  GL_FUNCTION_TRACE();
   if (pthread_getspecific(currentThreadOwnsItsWebGLContext)) {
     emscripten_glTexSubImage2D(target, level, xoffset, yoffset, width, height, format, type, pixels);
     return;
@@ -291,7 +307,7 @@ void glTexSubImage2D(GLenum target, GLint level, GLint xoffset, GLint yoffset, G
   if (!pixels || (sz >= 0 && sz < 256*1024)) { // run small buffer sizes asynchronously by copying - large buffers run synchronously
     void *ptr = memdup(pixels, sz);
     if (ptr || !pixels) {
-      emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIIIIIIII, &emscripten_glTexSubImage2D, ptr, target, level, xoffset, yoffset, width, height, format, type, ptr);
+      emscripten_dispatch_to_thread(GetCurrentTargetThread(), EM_FUNC_SIG_VIIIIIIIII, &emscripten_glTexSubImage2D, ptr, target, level, xoffset, yoffset, width, height, format, type, ptr);
       return;
     }
     // Fall through on allocation failure and run synchronously.
@@ -303,7 +319,7 @@ void glTexSubImage2D(GLenum target, GLint level, GLint xoffset, GLint yoffset, G
 ASYNC_GL_FUNCTION_2(EM_FUNC_SIG_VIF, void, glUniform1f, GLint, GLfloat);
 
 void glUniform1fv(GLint location, GLsizei count, const GLfloat *value) {
-  GL_FUNCTION_TRACE(__func__);
+  GL_FUNCTION_TRACE();
   if (pthread_getspecific(currentThreadOwnsItsWebGLContext)) {
     emscripten_glUniform1fv(location, count, value);
     return;
@@ -313,7 +329,7 @@ void glUniform1fv(GLint location, GLsizei count, const GLfloat *value) {
   if (sz < 256*1024) { // run small buffer sizes asynchronously by copying - large buffers run synchronously
     void *ptr = memdup(value, sz);
     if (ptr) {
-      emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform1fv, ptr, location, count, (GLfloat*)ptr);
+      emscripten_dispatch_to_thread(GetCurrentTargetThread(), EM_FUNC_SIG_VIII, &emscripten_glUniform1fv, ptr, location, count, (GLfloat*)ptr);
       return;
     }
     // Fall through on allocation failure and run synchronously.
@@ -325,7 +341,7 @@ void glUniform1fv(GLint location, GLsizei count, const GLfloat *value) {
 ASYNC_GL_FUNCTION_2(EM_FUNC_SIG_VII, void, glUniform1i, GLint, GLint);
 
 void glUniform1iv(GLint location, GLsizei count, const GLint *value) {
-  GL_FUNCTION_TRACE(__func__);
+  GL_FUNCTION_TRACE();
   if (pthread_getspecific(currentThreadOwnsItsWebGLContext)) {
     emscripten_glUniform1iv(location, count, value);
     return;
@@ -335,7 +351,7 @@ void glUniform1iv(GLint location, GLsizei count, const GLint *value) {
   if (sz < 256*1024) { // run small buffer sizes asynchronously by copying - large buffers run synchronously
     void *ptr = memdup(value, sz);
     if (ptr) {
-      emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform1iv, ptr, location, count, (GLint*)ptr);
+      emscripten_dispatch_to_thread(GetCurrentTargetThread(), EM_FUNC_SIG_VIII, &emscripten_glUniform1iv, ptr, location, count, (GLint*)ptr);
       return;
     }
     // Fall through on allocation failure and run synchronously.
@@ -346,7 +362,7 @@ void glUniform1iv(GLint location, GLsizei count, const GLint *value) {
 ASYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIFF, void, glUniform2f, GLint, GLfloat, GLfloat);
 
 void glUniform2fv(GLint location, GLsizei count, const GLfloat *value) {
-  GL_FUNCTION_TRACE(__func__);
+  GL_FUNCTION_TRACE();
   if (pthread_getspecific(currentThreadOwnsItsWebGLContext)) {
     emscripten_glUniform2fv(location, count, value);
     return;
@@ -356,7 +372,7 @@ void glUniform2fv(GLint location, GLsizei count, const GLfloat *value) {
   if (sz < 256*1024) { // run small buffer sizes asynchronously by copying - large buffers run synchronously
     void *ptr = memdup(value, sz);
     if (ptr) {
-      emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform2fv, ptr, location, count, (GLfloat*)ptr);
+      emscripten_dispatch_to_thread(GetCurrentTargetThread(), EM_FUNC_SIG_VIII, &emscripten_glUniform2fv, ptr, location, count, (GLfloat*)ptr);
       return;
     }
     // Fall through on allocation failure and run synchronously.
@@ -368,7 +384,7 @@ void glUniform2fv(GLint location, GLsizei count, const GLfloat *value) {
 ASYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIII, void, glUniform2i, GLint, GLint, GLint);
 
 void glUniform2iv(GLint location, GLsizei count, const GLint *value) {
-  GL_FUNCTION_TRACE(__func__);
+  GL_FUNCTION_TRACE();
   if (pthread_getspecific(currentThreadOwnsItsWebGLContext)) {
     emscripten_glUniform2iv(location, count, value);
     return;
@@ -378,7 +394,7 @@ void glUniform2iv(GLint location, GLsizei count, const GLint *value) {
   if (sz < 256*1024) { // run small buffer sizes asynchronously by copying - large buffers run synchronously
     void *ptr = memdup(value, sz);
     if (ptr) {
-      emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform2iv, ptr, location, count, (GLint*)ptr);
+      emscripten_dispatch_to_thread(GetCurrentTargetThread(), EM_FUNC_SIG_VIII, &emscripten_glUniform2iv, ptr, location, count, (GLint*)ptr);
       return;
     }
     // Fall through on allocation failure and run synchronously.
@@ -389,7 +405,7 @@ void glUniform2iv(GLint location, GLsizei count, const GLint *value) {
 ASYNC_GL_FUNCTION_4(EM_FUNC_SIG_VIFFF, void, glUniform3f, GLint, GLfloat, GLfloat, GLfloat);
 
 void glUniform3fv(GLint location, GLsizei count, const GLfloat *value) {
-  GL_FUNCTION_TRACE(__func__);
+  GL_FUNCTION_TRACE();
   if (pthread_getspecific(currentThreadOwnsItsWebGLContext)) {
     emscripten_glUniform3fv(location, count, value);
     return;
@@ -399,7 +415,7 @@ void glUniform3fv(GLint location, GLsizei count, const GLfloat *value) {
   if (sz < 256*1024) { // run small buffer sizes asynchronously by copying - large buffers run synchronously
     void *ptr = memdup(value, sz);
     if (ptr) {
-      emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform3fv, ptr, location, count, (GLfloat*)ptr);
+      emscripten_dispatch_to_thread(GetCurrentTargetThread(), EM_FUNC_SIG_VIII, &emscripten_glUniform3fv, ptr, location, count, (GLfloat*)ptr);
       return;
     }
     // Fall through on allocation failure and run synchronously.
@@ -411,7 +427,7 @@ void glUniform3fv(GLint location, GLsizei count, const GLfloat *value) {
 ASYNC_GL_FUNCTION_4(EM_FUNC_SIG_VIIII, void, glUniform3i, GLint, GLint, GLint, GLint);
 
 void glUniform3iv(GLint location, GLsizei count, const GLint *value) {
-  GL_FUNCTION_TRACE(__func__);
+  GL_FUNCTION_TRACE();
   if (pthread_getspecific(currentThreadOwnsItsWebGLContext)) {
     emscripten_glUniform3iv(location, count, value);
     return;
@@ -421,7 +437,7 @@ void glUniform3iv(GLint location, GLsizei count, const GLint *value) {
   if (sz < 256*1024) { // run small buffer sizes asynchronously by copying - large buffers run synchronously
     void *ptr = memdup(value, sz);
     if (ptr) {
-      emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform3iv, ptr, location, count, (GLint*)ptr);
+      emscripten_dispatch_to_thread(GetCurrentTargetThread(), EM_FUNC_SIG_VIII, &emscripten_glUniform3iv, ptr, location, count, (GLint*)ptr);
       return;
     }
     // Fall through on allocation failure and run synchronously.
@@ -432,7 +448,7 @@ void glUniform3iv(GLint location, GLsizei count, const GLint *value) {
 ASYNC_GL_FUNCTION_5(EM_FUNC_SIG_VIFFFF, void, glUniform4f, GLint, GLfloat, GLfloat, GLfloat, GLfloat);
 
 void glUniform4fv(GLint location, GLsizei count, const GLfloat *value) {
-  GL_FUNCTION_TRACE(__func__);
+  GL_FUNCTION_TRACE();
   if (pthread_getspecific(currentThreadOwnsItsWebGLContext)) {
     emscripten_glUniform4fv(location, count, value);
     return;
@@ -442,7 +458,7 @@ void glUniform4fv(GLint location, GLsizei count, const GLfloat *value) {
   if (sz < 256*1024) { // run small buffer sizes asynchronously by copying - large buffers run synchronously
     void *ptr = memdup(value, sz);
     if (ptr) {
-      emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform4fv, ptr, location, count, (GLfloat*)ptr);
+      emscripten_dispatch_to_thread(GetCurrentTargetThread(), EM_FUNC_SIG_VIII, &emscripten_glUniform4fv, ptr, location, count, (GLfloat*)ptr);
       return;
     }
     // Fall through on allocation failure and run synchronously.
@@ -454,7 +470,7 @@ void glUniform4fv(GLint location, GLsizei count, const GLfloat *value) {
 ASYNC_GL_FUNCTION_5(EM_FUNC_SIG_VIIIII, void, glUniform4i, GLint, GLint, GLint, GLint, GLint);
 
 void glUniform4iv(GLint location, GLsizei count, const GLint *value) {
-  GL_FUNCTION_TRACE(__func__);
+  GL_FUNCTION_TRACE();
   if (pthread_getspecific(currentThreadOwnsItsWebGLContext)) {
     emscripten_glUniform4iv(location, count, value);
     return;
@@ -464,7 +480,7 @@ void glUniform4iv(GLint location, GLsizei count, const GLint *value) {
   if (sz < 256*1024) { // run small buffer sizes asynchronously by copying - large buffers run synchronously
     void *ptr = memdup(value, sz);
     if (ptr) {
-      emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform4iv, ptr, location, count, (GLint*)ptr);
+      emscripten_dispatch_to_thread(GetCurrentTargetThread(), EM_FUNC_SIG_VIII, &emscripten_glUniform4iv, ptr, location, count, (GLint*)ptr);
       return;
     }
     // Fall through on allocation failure and run synchronously.
@@ -474,7 +490,7 @@ void glUniform4iv(GLint location, GLsizei count, const GLint *value) {
 }
 
 void glUniformMatrix2fv(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value) {
-  GL_FUNCTION_TRACE(__func__);
+  GL_FUNCTION_TRACE();
   if (pthread_getspecific(currentThreadOwnsItsWebGLContext)) {
     emscripten_glUniformMatrix2fv(location, count, transpose, value);
     return;
@@ -483,7 +499,7 @@ void glUniformMatrix2fv(GLint location, GLsizei count, GLboolean transpose, cons
   if (sz < 256*1024) { // run small buffer sizes asynchronously by copying - large buffers run synchronously
     void *ptr = memdup(value, sz);
     if (ptr) {
-      emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIII, &emscripten_glUniformMatrix2fv, ptr, location, count, transpose, (GLfloat*)ptr);
+      emscripten_dispatch_to_thread(GetCurrentTargetThread(), EM_FUNC_SIG_VIIII, &emscripten_glUniformMatrix2fv, ptr, location, count, transpose, (GLfloat*)ptr);
       return;
     }
     // Fall through on allocation failure and run synchronously.
@@ -493,7 +509,7 @@ void glUniformMatrix2fv(GLint location, GLsizei count, GLboolean transpose, cons
 }
 
 void glUniformMatrix3fv(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value) {
-  GL_FUNCTION_TRACE(__func__);
+  GL_FUNCTION_TRACE();
   if (pthread_getspecific(currentThreadOwnsItsWebGLContext)) {
     emscripten_glUniformMatrix3fv(location, count, transpose, value);
     return;
@@ -502,7 +518,7 @@ void glUniformMatrix3fv(GLint location, GLsizei count, GLboolean transpose, cons
   if (sz < 256*1024) { // run small buffer sizes asynchronously by copying - large buffers run synchronously
     void *ptr = memdup(value, sz);
     if (ptr) {
-      emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIII, &emscripten_glUniformMatrix3fv, ptr, location, count, transpose, (GLfloat*)ptr);
+      emscripten_dispatch_to_thread(GetCurrentTargetThread(), EM_FUNC_SIG_VIIII, &emscripten_glUniformMatrix3fv, ptr, location, count, transpose, (GLfloat*)ptr);
       return;
     }
     // Fall through on allocation failure and run synchronously.
@@ -512,7 +528,7 @@ void glUniformMatrix3fv(GLint location, GLsizei count, GLboolean transpose, cons
 }
 
 void glUniformMatrix4fv(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value) {
-  GL_FUNCTION_TRACE(__func__);
+  GL_FUNCTION_TRACE();
   if (pthread_getspecific(currentThreadOwnsItsWebGLContext)) {
     emscripten_glUniformMatrix4fv(location, count, transpose, value);
     return;
@@ -521,13 +537,13 @@ void glUniformMatrix4fv(GLint location, GLsizei count, GLboolean transpose, cons
   if (sz < 256*1024) { // run small buffer sizes asynchronously by copying - large buffers run synchronously
     void *ptr = memdup(value, sz);
     if (ptr) {
-      emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIII, &emscripten_glUniformMatrix4fv, ptr, location, count, transpose, (GLfloat*)ptr);
+      emscripten_dispatch_to_thread(GetCurrentTargetThread(), EM_FUNC_SIG_VIIIP, &emscripten_glUniformMatrix4fv, ptr, location, count, transpose, (GLfloat*)ptr);
       return;
     }
     // Fall through on allocation failure and run synchronously.
   }
 
-  emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIIII, &emscripten_glUniformMatrix4fv, location, count, transpose, value);
+  emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VIIIP, &emscripten_glUniformMatrix4fv, location, count, transpose, value);
 }
 
 ASYNC_GL_FUNCTION_1(EM_FUNC_SIG_VI, void, glUseProgram, GLuint);
@@ -543,9 +559,9 @@ VOID_SYNC_GL_FUNCTION_2(EM_FUNC_SIG_VII, void, glVertexAttrib4fv, GLuint, const
 
 // TODO: The following #define FULL_ES2 does not yet exist, we'll need to compile this file twice, for FULL_ES2 mode and without
 #if FULL_ES2
-VOID_SYNC_GL_FUNCTION_6(EM_FUNC_SIG_VIIIIII, void, glVertexAttribPointer, GLuint, GLint, GLenum, GLboolean, GLsizei, const void *);
+VOID_SYNC_GL_FUNCTION_6(EM_FUNC_SIG_PIIIIIP, void, glVertexAttribPointer, GLuint, GLint, GLenum, GLboolean, GLsizei, const void *);
 #else
-ASYNC_GL_FUNCTION_6(EM_FUNC_SIG_VIIIIII, void, glVertexAttribPointer, GLuint, GLint, GLenum, GLboolean, GLsizei, const void *);
+ASYNC_GL_FUNCTION_6(EM_FUNC_SIG_VIIIIIP, void, glVertexAttribPointer, GLuint, GLint, GLenum, GLboolean, GLsizei, const void *);
 #endif
 ASYNC_GL_FUNCTION_4(EM_FUNC_SIG_VIIII, void, glViewport, GLint, GLint, GLsizei, GLsizei);
 
@@ -563,6 +579,8 @@ RET_SYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIII, void, glGetQueryObjectui64vEXT, GLenum,
 
 #endif // ~(__EMSCRIPTEN_PTHREADS__ && __EMSCRIPTEN_OFFSCREEN_FRAMEBUFFER__)
 
+#if GL_ENABLE_GET_PROC_ADDRESS
+
 // Returns a function pointer to the given WebGL 1 extension function, when queried without
 // a GL extension suffix such as "EXT", "OES", or "ANGLE". This function is used by
 // emscripten_GetProcAddress() to implement legacy GL emulation semantics for portability.
@@ -781,3 +799,5 @@ void *emscripten_webgl1_get_proc_address(const char *name) {
 
   return 0;
 }
+
+#endif
