@@ -344,15 +344,14 @@ def generate_object_file(data_files):
       .dc.a 0
       .size __emscripten_embedded_file_data, {total_size}
       '''))
+  cmd = [shared.EMCC, '-c', asm_file, '-o', options.obj_output]
   if options.wasm64:
     target = 'wasm64-unknown-emscripten'
+    cmd.append('-Wno-experimental')
   else:
     target = 'wasm32-unknown-emscripten'
-  shared.check_call([shared.EMCC,
-                     '-c',
-                     '--target=' + target,
-                     '-o', options.obj_output,
-                     asm_file])
+  cmd.append('--target=' + target)
+  shared.check_call(cmd)
 
 
 def main():
