@@ -4,13 +4,16 @@
 # found in the LICENSE file.
 
 import os
-import logging
 
-TAG = '1.6.37'
-HASH = '2ce2b855af307ca92a6e053f521f5d262c36eb836b4810cb53c809aa3ea2dcc08f834aee0ffd66137768a54397e28e92804534a74abb6fc9f6f3127f14c9c338'
+TAG = '1.6.39'
+HASH = '19851afffbe2ffde62d918f7e9017dec778a7ce9c60c75cdc65072f086e6cdc9d9895eb7b207535a84cb5f4ead77ebc2aa9d80025f153662903023e1f7ab9bae'
 
 deps = ['zlib']
-variants = {'libpng-mt': {'USE_PTHREADS': 1}}
+variants = {
+  'libpng-mt': {'PTHREADS': 1},
+  'libpng-wasm-sjlj': {'SUPPORT_LONGJMP': 'wasm'},
+  'libpng-mt-wasm-sjlj': {'PTHREADS': 1, 'SUPPORT_LONGJMP': 'wasm'},
+}
 
 
 def needed(settings):
@@ -18,7 +21,12 @@ def needed(settings):
 
 
 def get_lib_name(settings):
-  return 'libpng' + ('-mt' if settings.USE_PTHREADS else '') + '.a'
+  suffix = ''
+  if settings.PTHREADS:
+    suffix += '-mt'
+  if settings.SUPPORT_LONGJMP == 'wasm':
+    suffix += '-wasm-sjlj'
+  return f'libpng{suffix}.a'
 
 
 def get(ports, settings, shared):
@@ -26,15 +34,15 @@ def get(ports, settings, shared):
   ports.fetch_project('libpng', f'https://storage.googleapis.com/webassembly/emscripten-ports/libpng-{TAG}.tar.gz', sha512hash=HASH)
 
   def create(final):
-    logging.info('building port: libpng')
-
     source_path = os.path.join(ports.get_dir(), 'libpng', 'libpng-' + TAG)
     ports.write_file(os.path.join(source_path, 'pnglibconf.h'), pnglibconf_h)
     ports.install_headers(source_path)
 
     flags = ['-sUSE_ZLIB']
-    if settings.USE_PTHREADS:
-      flags += ['-sUSE_PTHREADS']
+    if settings.PTHREADS:
+      flags += ['-pthread']
+    if settings.SUPPORT_LONGJMP == 'wasm':
+      flags.append('-sSUPPORT_LONGJMP=wasm')
 
     ports.build_port(source_path, final, 'libpng', flags=flags, exclude_files=['pngtest'], exclude_dirs=['scripts', 'contrib'])
 
@@ -59,9 +67,9 @@ def show():
 
 pnglibconf_h = r'''/* pnglibconf.h - library build configuration */
 
-/* libpng version 1.6.37 */
+/* libpng version 1.6.39 */
 
-/* Copyright (c) 2018-2019 Cosmin Truta */
+/* Copyright (c) 2018-2022 Cosmin Truta */
 /* Copyright (c) 1998-2002,2004,2006-2018 Glenn Randers-Pehrson */
 
 /* This code is released under the libpng license. */
@@ -333,7 +341,7 @@ be appreciated.
 #define PNG_USER_MEM_SUPPORTED
 #define PNG_USER_TRANSFORM_INFO_SUPPORTED
 #define PNG_USER_TRANSFORM_PTR_SUPPORTED
-/*#undef PNG_WARNINGS_SUPPORTED*/
+#undef PNG_WARNINGS_SUPPORTED
 #define PNG_WRITE_16BIT_SUPPORTED
 #define PNG_WRITE_ANCILLARY_CHUNKS_SUPPORTED
 #define PNG_WRITE_BGR_SUPPORTED
