@@ -471,7 +471,9 @@ class Library:
     cflags = self.get_cflags()
     if self.deterministic_paths:
       source_dir = utils.path_from_root()
+      relative_source_dir = os.path.relpath(source_dir, build_dir)
       cflags += [f'-ffile-prefix-map={source_dir}=/emsdk/emscripten',
+                 f'-ffile-prefix-map={relative_source_dir}/=',
                  '-fdebug-compilation-dir=/emsdk/emscripten']
     asflags = get_base_cflags(preprocess=False)
     input_files = self.get_files()
@@ -491,7 +493,9 @@ class Library:
     cflags = self.get_cflags()
     if self.deterministic_paths:
       source_dir = utils.path_from_root()
+      relative_source_dir = os.path.relpath(source_dir, build_dir)
       cflags += [f'-ffile-prefix-map={source_dir}=/emsdk/emscripten',
+                 f'-ffile-prefix-map={relative_source_dir}/=',
                  '-fdebug-compilation-dir=/emsdk/emscripten']
     case_insensitive = is_case_insensitive(build_dir)
     for src in self.get_files():
@@ -531,6 +535,7 @@ class Library:
         # Use relative paths to reduce the length of the command line.
         # This allows to avoid switching to a response file as often.
         src = os.path.relpath(src, build_dir)
+        src = utils.normalize_path(src)
         batches.setdefault(tuple(cmd), []).append(src)
       objects.add(o)
 
@@ -914,7 +919,7 @@ class libcompiler_rt(MTLibrary, SjLjLibrary):
   src_dir = 'system/lib/compiler-rt/lib/builtins'
   includes = ['system/lib/libc']
   # gcc_personality_v0.c depends on libunwind, which don't include by default.
-  src_files = glob_in_path(src_dir, '*.c', excludes=['gcc_personality_v0.c', 'truncdfbf2.c', 'truncsfbf2.c'])
+  src_files = glob_in_path(src_dir, '*.c', excludes=['gcc_personality_v0.c', 'truncdfbf2.c', 'truncsfbf2.c', 'crtbegin.c', 'crtend.c'])
   src_files += files_in_path(
       path='system/lib/compiler-rt',
       filenames=[
@@ -1220,6 +1225,7 @@ class libc(MuslInternalLibrary,
           'sigaddset.c',
           'sigdelset.c',
           'sigemptyset.c',
+          'sigisemptyset.c',
           'sigfillset.c',
           'sigismember.c',
           'siginterrupt.c',
@@ -1599,7 +1605,8 @@ class libcxx(NoExceptLibrary, MTLibrary):
     'locale_win32.cpp',
     'thread_win32.cpp',
     'support.cpp',
-    'int128_builtins.cpp'
+    'int128_builtins.cpp',
+    'libdispatch.cpp',
   ]
 
 
@@ -1647,7 +1654,7 @@ class libmalloc(MTLibrary):
 
   def __init__(self, **kwargs):
     self.malloc = kwargs.pop('malloc')
-    if self.malloc not in ('dlmalloc', 'emmalloc', 'emmalloc-debug', 'emmalloc-memvalidate', 'emmalloc-verbose', 'emmalloc-memvalidate-verbose', 'none'):
+    if self.malloc not in ('dlmalloc', 'emmalloc', 'emmalloc-debug', 'emmalloc-memvalidate', 'emmalloc-verbose', 'emmalloc-memvalidate-verbose', 'mimalloc', 'none'):
       raise Exception('malloc must be one of "emmalloc[-debug|-memvalidate][-verbose]", "dlmalloc" or "none", see settings.js')
 
     self.use_errno = kwargs.pop('use_errno')
@@ -1696,7 +1703,7 @@ class libmalloc(MTLibrary):
     return name
 
   def can_use(self):
-    return super().can_use() and settings.MALLOC != 'none'
+    return super().can_use() and settings.MALLOC != 'none' and settings.MALLOC != 'mimalloc'
 
   @classmethod
   def vary_on(cls):
@@ -1724,6 +1731,48 @@ class libmalloc(MTLibrary):
             [dict(malloc='emmalloc-verbose', **combo) for combo in combos if combo['verbose'] and not combo['memvalidate']])
 
 
+class libmimalloc(MTLibrary):
+  name = 'libmimalloc'
+
+  cflags = [
+    '-fno-builtin',
+    '-Wno-deprecated-pragma',
+    # build emmalloc as only a system allocator, without exporting itself onto
+    # malloc/free in the global scope
+    '-DEMMALLOC_NO_STD_EXPORTS',
+    # build mimalloc with an override of malloc/free
+    '-DMI_MALLOC_OVERRIDE',
+    # TODO: add build modes that include debug checks 1,2,3
+    '-DMI_DEBUG=0',
+  ]
+
+  # malloc/free/calloc are runtime functions and can be generated during LTO
+  # Therefor they cannot themselves be part of LTO.
+  force_object_files = True
+
+  includes = ['system/lib/mimalloc/include']
+
+  # Build all of mimalloc, and also emmalloc which is used as the system
+  # allocator underneath it.
+  src_dir = 'system/lib/'
+  src_files = glob_in_path(
+    path='system/lib/mimalloc/src',
+    glob_pattern='*.c',
+    # mimalloc includes some files at the source level, so exclude them here.
+    excludes=['alloc-override.c', 'page-queue.c', 'static.c']
+  )
+  src_files += files_in_path(
+    path='system/lib/mimalloc/src/prim',
+    filenames=['prim.c']
+  )
+  src_files += files_in_path(
+    path='system/lib/',
+    filenames=['emmalloc.c'])
+
+  def can_use(self):
+    return super().can_use() and settings.MALLOC == 'mimalloc'
+
+
 class libal(Library):
   name = 'libal'
 
@@ -1736,7 +1785,7 @@ class libGL(MTLibrary):
   name = 'libGL'
 
   src_dir = 'system/lib/gl'
-  src_files = ['gl.c', 'webgl1.c', 'libprocaddr.c']
+  src_files = ['gl.c', 'webgl1.c', 'libprocaddr.c', 'webgl2.c']
 
   cflags = ['-Oz', '-fno-inline-functions']
 
@@ -1745,10 +1794,7 @@ class libGL(MTLibrary):
     self.is_webgl2 = kwargs.pop('is_webgl2')
     self.is_ofb = kwargs.pop('is_ofb')
     self.is_full_es3 = kwargs.pop('is_full_es3')
-    if self.is_webgl2 or self.is_full_es3:
-      # Don't use append or += here, otherwise we end up adding to
-      # the class member.
-      self.src_files = self.src_files + ['webgl2.c']
+    self.is_enable_get_proc_address = kwargs.pop('is_enable_get_proc_address')
     super().__init__(**kwargs)
 
   def get_base_name(self):
@@ -1761,23 +1807,26 @@ class libGL(MTLibrary):
       name += '-ofb'
     if self.is_full_es3:
       name += '-full_es3'
+    if self.is_enable_get_proc_address:
+      name += '-getprocaddr'
     return name
 
   def get_cflags(self):
     cflags = super().get_cflags()
     if self.is_legacy:
       cflags += ['-DLEGACY_GL_EMULATION=1']
-    if self.is_webgl2:
-      cflags += ['-DMAX_WEBGL_VERSION=2']
+    cflags += [f'-DMAX_WEBGL_VERSION={2 if self.is_webgl2 else 1}']
     if self.is_ofb:
       cflags += ['-D__EMSCRIPTEN_OFFSCREEN_FRAMEBUFFER__']
     if self.is_full_es3:
       cflags += ['-D__EMSCRIPTEN_FULL_ES3__']
+    if self.is_enable_get_proc_address:
+      cflags += ['-DGL_ENABLE_GET_PROC_ADDRESS=1']
     return cflags
 
   @classmethod
   def vary_on(cls):
-    return super().vary_on() + ['is_legacy', 'is_webgl2', 'is_ofb', 'is_full_es3']
+    return super().vary_on() + ['is_legacy', 'is_webgl2', 'is_ofb', 'is_full_es3', 'is_enable_get_proc_address']
 
   @classmethod
   def get_default_variation(cls, **kwargs):
@@ -1786,6 +1835,7 @@ class libGL(MTLibrary):
       is_webgl2=settings.MAX_WEBGL_VERSION >= 2,
       is_ofb=settings.OFFSCREEN_FRAMEBUFFER,
       is_full_es3=settings.FULL_ES3,
+      is_enable_get_proc_address=settings.GL_ENABLE_GET_PROC_ADDRESS,
       **kwargs
     )
 
@@ -1808,6 +1858,7 @@ class libembind(Library):
 
   def get_cflags(self):
     cflags = super().get_cflags()
+    cflags.append('-std=c++20')
     if not self.with_rtti:
       cflags += ['-fno-rtti', '-DEMSCRIPTEN_HAS_UNBOUND_TYPE_NAMES=0']
     return cflags
@@ -2242,7 +2293,9 @@ def get_libs_to_link(args, forced, only_forced):
     if not settings.EXIT_RUNTIME:
       add_library('libnoexit')
     add_library('libc')
-    if settings.MALLOC != 'none':
+    if settings.MALLOC == 'mimalloc':
+      add_library('libmimalloc')
+    elif settings.MALLOC != 'none':
       add_library('libmalloc')
   add_library('libcompiler_rt')
   if settings.LINK_AS_CXX:
